==> 0.6.2.tar.gz: lib/src/model/mirrors.dart <==
part of fields.model;

final _listClassMirror = reflectClass(List);
final _mapClassMirror = reflectClass(Map);
final _objectClassMirror = reflectClass(Object);
final _dynamicTypeMirror = currentMirrorSystem().dynamicType;

Type _typeOf(TypeMirror tm) {
  if (tm == _dynamicTypeMirror || tm is! ClassMirror) {
    return dynamic;
  } else {
    return (tm as ClassMirror).reflectedType;
  }
}

bool _isSimpleType(Type t) {
  if ( t == bool
    || t == double
    || t == int
    || t == List
    || t == Map
    || t == Object
    || t == num
    || t == String
    || t == Type ) {
    return true;      
  } else {
    if (dynamic == t) {
      return true;
    } else {
      ClassMirror cm = reflectClass(t);
      if (_isAssignableFrom(_listClassMirror, cm) ||
          _isAssignableFrom(_mapClassMirror, cm)) {
        return true;
      }
    }
    return false;
  }
}

/// Returns `true` of the given [type] and the given [other] represent the same
/// type, or if the [type] is in the inheritance hierarchy of [other], or if
/// [other] implements the interface of [type]; else returns `false`.
bool _isAssignableFrom(TypeMirror type, TypeMirror other) {
  assert(type != null);
  assert(other != null);
  if (type == _dynamicTypeMirror) return true;
  if (!other.isOriginalDeclaration) other = other.originalDeclaration;  
  if (other == type) return true;
  if (other is! ClassMirror) return false;
  ClassMirror otherCm = other;  
  if (otherCm.superclass == null) return type == _objectClassMirror;
  if (otherCm.superclass == type) return true;
  if (otherCm.superinterfaces.any((i) => i == type)) return true;
  return _isAssignableFrom(type, otherCm.superclass);
}

==> 0.7.2.tar.gz: lib/src/model/mirrors.dart <==
part of fields.model;

final _listClassMirror = reflectClass(List);
final _setClassMirror = reflectClass(Set);
final _mapClassMirror = reflectClass(Map);
final _objectClassMirror = reflectClass(Object);
final _dynamicTypeMirror = currentMirrorSystem().dynamicType;

Type _typeOf(TypeMirror tm) {
  if (tm == _dynamicTypeMirror || tm is! ClassMirror) {
    return dynamic;
  } else {
    return (tm as ClassMirror).reflectedType;
  }
}

const List<Type> _atomicTypes = const [ 
    bool, double, int, List, Map, Set, Object, num, String, Type ];

bool _isAtomicType(Type t) {
  if (atomicTypes.contains(t)) {
    return true;      
  } else {
    if (dynamic == t) {
      return true;
    } else {
      ClassMirror cm = reflectClass(t);
      if (_isAssignableFrom(_listClassMirror, cm) ||
          _isAssignableFrom(_mapClassMirror, cm) ||
          _isAssignableFrom(_setClassMirror, cm)) {
        return true;
      }
    }
    return false;
  }
}

/// Returns `true` of the given [type] and the given [other] represent the same
/// type, or if the [type] is in the inheritance hierarchy of [other], or if
/// [other] implements the interface of [type]; else returns `false`.
bool _isAssignableFrom(TypeMirror type, TypeMirror other) {
  assert(type != null);
  assert(other != null);
  if (type == _dynamicTypeMirror) return true;
  if (!other.isOriginalDeclaration) other = other.originalDeclaration;  
  if (other == type) return true;
  if (other is! ClassMirror) return false;
  ClassMirror otherCm = other;  
  if (otherCm.superclass == null) return type == _objectClassMirror;
  if (otherCm.superclass == type) return true;
  if (otherCm.superinterfaces.any((i) => i == type)) return true;
  return _isAssignableFrom(type, otherCm.superclass);
}
